import functools
import inspect
import logging
import os
import shutil
import warnings
from collections.abc import Iterable
from dataclasses import asdict
from typing import Any

import torch_geometric.graphgym.register as register
from torch_geometric.data.makedirs import makedirs

try:  # Define global config object
    from yacs.config import CfgNode as CN

    eixgnn_cfg = CN()
except ImportError:
    eixgnn_cfg = None
    warnings.warn(
        "Could not define global config object. Please install "
        "'yacs' for using the GraphGym experiment manager via "
        "'pip install yacs'."
    )


def set_eixgnn_cfg(eixgnn_cfg):
    r"""
    This function sets the default config value.
    1) Note that for an experiment, only part of the arguments will be used
    The remaining unused arguments won't affect anything.
    So feel free to register any argument in graphgym.contrib.config
    2) We support *at most* two levels of configs, e.g., eixgnn_cfg.dataset.name
    :return: configuration use by the experiment.
    """
    if eixgnn_cfg is None:
        return eixgnn_cfg

    eixgnn_cfg.seed = 0
    eixgnn_cfg.L = 50
    eixgnn_cfg.p = 0.5
    eixgnn_cfg.importance_sampling_strategy = "node"
    eixgnn_cfg.domain_similarity = "relative_edge_density"
    eixgnn_cfg.signal_similarity = "KL"
    eixgnn_cfg.shapley_value_approx = 100


def assert_eixgnn_cfg(eixgnn_cfg):
    r"""Checks config values, do necessary post processing to the configs"""
    if not (0 <= eixgnn_cfg.p and eixgnn_cfg.p <= 1):
        raise ValueError("p needs to be between 0 and 1")


set_eixgnn_cfg(eixgnn_cfg)

#  def dump_eixgnn_cfg(eixgnn_cfg,path):
#  r"""
#  Dumps the config to the output directory specified in
#  :obj:`eixgnn_cfg.out_dir`
#  Args:
#  eixgnn_cfg (CfgNode): Configuration node
#  """
#  makedirs(eixgnn_cfg.out_dir)
#  eixgnn_cfg_file = os.path.join(eixgnn_cfg.out_dir, eixgnn_cfg.eixgnn_cfg_dest)
#  with open(eixgnn_cfg_file, "w") as f:
#  eixgnn_cfg.dump(stream=f)
#
#
def load_eixgnn_cfg(eixgnn_cfg, args):
    r"""
    Load configurations from file system and command line
    Args:
    eixgnn_cfg (CfgNode): Configuration node
    args (ArgumentParser): Command argument parser
    """
    eixgnn_cfg.merge_from_file(args.eixgnn_cfg_file)
    eixgnn_cfg.merge_from_list(args.opts)
    assert_eixgnn_cfg(eixgnn_cfg)


#
#  def makedirs_rm_exist(dir):
#  if os.path.isdir(dir):
#  shutil.rmtree(dir)
#  os.makedirs(dir, exist_ok=True)
#
#
#  def get_fname(fname):
#  r"""
#  Extract filename from file name path
#  Args:
#  fname (string): Filename for the yaml format configuration file
#  """
#  fname = fname.split("/")[-1]
#  if fname.endswith(".yaml"):
#  fname = fname[:-5]
#  elif fname.endswith(".yml"):
#  fname = fname[:-4]
#  return fname
#
#
#  def set_out_dir(out_dir, fname):
#  r"""
#  Create the directory for full experiment run
#  Args:
#  out_dir (string): Directory for output, specified in :obj:`eixgnn_cfg.out_dir`
#  fname (string): Filename for the yaml format configuration file
#  """
#  fname = get_fname(fname)
#  eixgnn_cfg.out_dir = os.path.join(out_dir, fname)
#  Make output directory
#  if eixgnn_cfg.train.auto_resume:
#  os.makedirs(eixgnn_cfg.out_dir, exist_ok=True)
#  else:
#  makedirs_rm_exist(eixgnn_cfg.out_dir)
#
#
#  def set_run_dir(out_dir):
#  r"""
#  Create the directory for each random seed experiment run
#  Args:
#  out_dir (string): Directory for output, specified in :obj:`eixgnn_cfg.out_dir`
#  fname (string): Filename for the yaml format configuration file
#  """
#  eixgnn_cfg.run_dir = os.path.join(out_dir, str(eixgnn_cfg.seed))
#  Make output directory
#  if eixgnn_cfg.train.auto_resume:
#  os.makedirs(eixgnn_cfg.run_dir, exist_ok=True)
#  else:
#  makedirs_rm_exist(eixgnn_cfg.run_dir)
#
#
#  def from_config(func):
#  if inspect.isclass(func):
#  params = list(inspect.signature(func.__init__).parameters.values())[1:]
#  else:
#  params = list(inspect.signature(func).parameters.values())
#
#  arg_names = [p.name for p in params]
#  has_defaults = [p.default != inspect.Parameter.empty for p in params]
#
#  @functools.wraps(func)
#  def wrapper(*args, eixgnn_cfg: Any = None, **kwargs):
#  if eixgnn_cfg is not None:
#  eixgnn_cfg = (
#  dict(eixgnn_cfg)
#  if isinstance(eixgnn_cfg, Iterable)
#  else asdict(eixgnn_cfg)
#  )
#
#  iterator = zip(arg_names[len(args) :], has_defaults[len(args) :])
#  for arg_name, has_default in iterator:
#  if arg_name in kwargs:
#  continue
#  elif arg_name in eixgnn_cfg:
#  kwargs[arg_name] = eixgnn_cfg[arg_name]
#  elif not has_default:
#  raise ValueError(f"'eixgnn_cfg.{arg_name}' undefined")
#  return func(*args, **kwargs)
#
#  return wrapper
